/*
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of ac32wav.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file
*/

#include "ac3_format.h"

START_M2W_NAMESPACE

const binary AC3_FORMAT_chunk::Ac3GUID[16] =
{0x2C, 0x80, 0x6D, 0xE0, 0x46, 0xDB, 0xCF, 0x11, 0xB4, 0xD1, 0x00, 0x80, 0x5F, 0x6C, 0xBB, 0xEA};

uint32 AC3_FORMAT_chunk::Write(IO_file & aFile)
{
    FORMAT_chunk::Write(aFile);

    double Block_size = mySampleSize;

    lil_uint16 lu16;
    lu16 = myFormatEx.Samples.wSamplesPerBlock;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lil_uint32 lu32;
    lu32 = myFormatEx.dwChannelMask;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    aFile.Write(myFormatEx.SubFormat, 16);

    return FormatTagSize();
}

#if 0
uint32 MP3_FORMAT_chunk::Write(IO_file & aFile)
{
    FORMAT_chunk::Write(aFile);

    double Block_size = mySampleSize;

    myFormatEx.nBlockSize = uint16(0.5 + (Block_size * myFormat.nAvgBytesPerSec / myFormat.nSamplesPerSec));

    lil_uint16 lu16;
    lu16 = myFormatEx.wID;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lil_uint32 lu32;
    lu32 = myFormatEx.fdwFlags;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    lu16 = myFormatEx.nBlockSize;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.nFramesPerBlock;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.nCodecDelay;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    return FormatTagSize();
}

/**
    \todo support original, copyright, private and protection bits
*/
uint32 MPEG1_FORMAT_chunk::Write(IO_file & aFile)
{
    FORMAT_chunk::Write(aFile);

/*    switch (myLayer) {
    case MPEG_LAYER_1:
        myFormatEx.fwHeadLayer = ACM_MPEG_LAYER1;
	break;
    case MPEG_LAYER_2:
        myFormatEx.fwHeadLayer = ACM_MPEG_LAYER2;
	break;
    case MPEG_LAYER_3:
        myFormatEx.fwHeadLayer = ACM_MPEG_LAYER3;
	break;
    default:
        myFormatEx.fwHeadLayer = 0;
	break;
    }*/

    if (!myCBR)
	myFormatEx.dwHeadBitrate = 0;
    else
	myFormatEx.dwHeadBitrate = myBitRate;

/*    switch (myChannels)
    {
    case MPEG_CHANNEL_STEREO:
	myFormatEx.fwHeadMode = ACM_MPEG_STEREO;
	break;
    case MPEG_CHANNEL_JOINT_STEREO:
	myFormatEx.fwHeadMode = ACM_MPEG_JOINTSTEREO;
	break;
    case MPEG_CHANNEL_DUAL_CHANNEL:
	myFormatEx.fwHeadMode = ACM_MPEG_DUALCHANNEL;
	break;
    case MPEG_CHANNEL_MONO:
	myFormatEx.fwHeadMode = ACM_MPEG_SINGLECHANNEL;
	break;
    default: // to make g++ happy
	myFormatEx.fwHeadMode = ACM_MPEG_STEREO;
	break;
    }*/

/*    switch (myEmphasis)
    {
    case MPEG_EMPH_50_15MS:
	myFormatEx.wHeadEmphasis = ACM_MPEG_50_15MS;
	break;
    case MPEG_EMPH_RESERVED:
	myFormatEx.wHeadEmphasis = ACM_MPEG_RESERVED;
	break;
    case MPEG_EMPH_CCITT_J17:
	myFormatEx.wHeadEmphasis = ACM_MPEG_CCITT_J17;
	break;
    default:
	myFormatEx.wHeadEmphasis = ACM_MPEG_NO_EMPHASIS;
	break;
    }*/

    if (myFormat.nSamplesPerSec >= 32000)
	myFormatEx.fwHeadFlags = ACM_MPEG_ID_MPEG1;
    else
	myFormatEx.fwHeadFlags = 0;
    
    if (myPrivate)
	myFormatEx.fwHeadFlags |= ACM_MPEG_PRIVATEBIT;

    if (myCopyright)
	myFormatEx.fwHeadFlags |= ACM_MPEG_COPYRIGHT;

    if (myOriginal)
	myFormatEx.fwHeadFlags |= ACM_MPEG_ORIGINALHOME;

    if (myProtection)
	myFormatEx.fwHeadFlags |= ACM_MPEG_PROTECTIONBIT;

    lil_uint16 lu16;
    lu16 = myFormatEx.fwHeadLayer;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lil_uint32 lu32;
    lu32 = myFormatEx.dwHeadBitrate;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    lu16 = myFormatEx.fwHeadMode;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.fwHeadModeExt;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.wHeadEmphasis;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu16 = myFormatEx.fwHeadFlags;
    aFile.Write(&lu16.endian(), sizeof(uint16));

    lu32 = myFormatEx.dwPTSLow;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    lu32 = myFormatEx.dwPTSHigh;
    aFile.Write(&lu32.endian(), sizeof(uint32));

    return FormatTagSize();
}
#endif // 0

void AC3_FORMAT_chunk::setChannelMode(const Ac3_Channel mode)
{
    switch (mode)
    {
    case AC3_1_AND_1:
	myFormatEx.dwChannelMask = SPEAKER_SIDE_LEFT | SPEAKER_SIDE_RIGHT;
	break;
    case AC3_CENTER:
	myFormatEx.dwChannelMask = SPEAKER_FRONT_CENTER;
	break;
    case AC3_LEFT_RIGHT:
	myFormatEx.dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT;
	break;
    case AC3_LEFT_CENTER_RIGHT:
	myFormatEx.dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_CENTER | SPEAKER_FRONT_RIGHT;
	break;
    case AC3_LEFT_RIGHT_SURROUND:
	myFormatEx.dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_RIGHT | SPEAKER_LOW_FREQUENCY;
	break;
    case AC3_LEFT_CENTER_RIGHT_SURROUND:
	myFormatEx.dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_FRONT_CENTER | SPEAKER_FRONT_RIGHT | SPEAKER_LOW_FREQUENCY;
	break;
    case AC3_LEFT_SURROUND_RIGHT_SURROUND:
	myFormatEx.dwChannelMask = SPEAKER_FRONT_LEFT | SPEAKER_BACK_LEFT | SPEAKER_FRONT_RIGHT | SPEAKER_BACK_RIGHT;
	break;
    case AC3_CENTER_LEFT_SURROUND_RIGHT_SURROUND:
	myFormatEx.dwChannelMask = SPEAKER_FRONT_CENTER | SPEAKER_FRONT_LEFT | SPEAKER_BACK_LEFT | SPEAKER_FRONT_RIGHT | SPEAKER_BACK_RIGHT;
	break;
    }
}

END_M2W_NAMESPACE
